import Move from './model/move';
import { IPiece, IJSONKifuFormat, IMoveMoveFormat } from 'json-kifu-format/src/Formats';
export default class Editor {
    private _currentNum;
    /** 特定の指し手における盤面などの情報 */
    private _field;
    /** 現在操作中の棋譜に関する情報 */
    private readonly;
    private moveData;
    private _header;
    private initial;
    private preset;
    /**
     * jkfを渡して初期化
     * @param jkf
     * @param readonly
     */
    constructor(jkf?: IJSONKifuFormat, readonly?: boolean);
    readonly moves: Array<Move>;
    /**
     * 最後に指された手の移動情報を表示する
     */
    readonly lastMove: Move;
    readonly comment: string[] | null;
    readonly board: IPiece[][];
    readonly reverseBoard: IPiece[][];
    readonly hands: Array<{
        [index: string]: number;
    }>;
    readonly nextMoves: Array<Move>;
    readonly nextSelect: number;
    /**
     * 最後に指したプレイヤー
     */
    readonly color: number;
    /**
     * 現在の盤面が棋譜の何番目の指し手のものか表示する
     */
    /**
    * 指し手を進む/戻す
    */
    currentNum: number;
    /**
     * 現在の盤面が次の指し手候補を複数持つかどうかを返す
     */
    readonly isFork: boolean;
    /**
     * jkfのヘッダー情報を返す
     */
    readonly header: {
        [index: string]: string;
    };
    /**
     * 現在の盤面の駒を取得する
     *
     * @param kx 盤面のX座標 7六歩の7
     * @param ky 盤面のY座標 7六歩の六
     */
    getBoardPiece(x: number, y: number): IPiece;
    /**
     * 現在の指し手にコメントを追加する
     * @param comment
     */
    addComment(comment: string): void;
    /**
     * 現在の指し手のコメントをリセットする
     */
    resetComment(): void;
    /**
     * jkfをエクスポートする
     */
    export(): IJSONKifuFormat;
    /**
     * 選択した指し手番号へ移動する
     *
     * @param moveNum
     */
    go(newNum: number): void;
    readonly currentMove: Move;
    dispCurrentInfo(): string;
    /**
     * 現在の盤面を見やすい形で表した文字列を返す
     */
    dispBoard(): string;
    /**
     * 現在の棋譜を返す
     */
    dispKifuMoves(): string;
    /**
     * 次の指し手候補を返す
     */
    dispNextMoves(): string;
    /**
     * 先手の持ち駒を返す
     */
    dispHand(player: number): string;
    /**
     * 現在の指し手を削除する
     */
    deleteMove(deleteNum: number): void;
    /**
     * 受け取った指し手情報を次の手として追加する
     * 現在の指し手が最後の指し手でない場合は分岐指し手として追加する
     *
     * @param moveInfoObj 指し手情報オブジェクト
     * @param comment コメント
     */
    addMovefromObj(moveInfoObj: IMoveMoveFormat, comment?: Array<string> | string | null): void;
    /**
     * 与えられた盤面の駒を移動する指し手を次の手として追加する
     *
     * @param fromX
     * @param fromY
     * @param toX
     * @param toY
     * @param promote
     * @param comment
     */
    addBoardMove(fromX: number, fromY: number, toX: number, toY: number, promote?: boolean, comment?: Array<string> | string | null): void;
    /**
     * 与えられた持ち駒から盤面に駒を置く指し手を次の手として追加する
     *
     * @param komaNum
     * @param toX
     * @param toY
     * @param comment
     */
    addHandMove(komaString: string, toX: number, toY: number, comment?: Array<string> | string | null): void;
    /**
     * 指定された盤上駒の移動可能なMoveを返す
     * @param fromX
     * @param fromY
     */
    getKomaMoves(fromX: number, fromY: number, reverse?: boolean): Array<Array<number>>;
    /**
     * 移動可能な駒の座標を返す
     */
    getMovables(reverse?: boolean): any[][];
    getPutables(komaString: string, reverse?: boolean): any[][];
    /**
     * 指定番号の移動が分岐をもつかどうか
     *
     * @param num
     */
    haveFork(num: number): boolean;
    /**
     * 指し手の分岐を切り替える
     *
     * @param forkIndex
     */
    switchFork(forkIndex: number): void;
    /**
     * 指定された指し手の分岐を削除する
     *
     * @param forkIndex
     */
    deleteFork(forkIndex: number): void;
    /**
     * 指定された2つの分岐の順番を入れ替える
     *
     * @param forkIndex1
     * @param forkIndex2
     */
    swapFork(forkIndex1: number, forkIndex2: number): void;
    /**
     * jkfヘッダーに指定のパラメータを追加する
     *
     * @param key
     * @param value
     */
    addInfo(key: string, value: string): void;
    /**
     * jkfヘッダーの指定パラメータを削除する
     * @param key
     */
    deleteInfo(key: string): void;
    /**
     * jkfオブジェクトをロードする
     * @param jkf
     */
    load(jkf: IJSONKifuFormat): void;
    private isEditable;
    /**
     * 指し手オブジェクトを作成する(持ち駒から配置される場合のためにkomaStringを入力する)
     *
     * @param komaString
     * @param fromX
     * @param fromY
     * @param toX
     * @param toY
     * @param promote
     */
    private makeMoveData;
}
