import Move from './move';
import Pos from './pos';
import { IPiece, IStateFormat } from 'json-kifu-format/src/Formats';
export default class Field {
    private _board;
    private _reverseBoard;
    private _hands;
    private _color;
    private _nomove;
    private _initBoard;
    private _initHands;
    private _initColor;
    constructor(board: Array<Array<Object>>, hands?: Array<{
        [index: string]: number;
    }>, color?: number);
    applyMove(move: Move): void;
    rewindMove(move: Move): void;
    /**
     * from座標の駒がto座標の位置に移動できるか判定する(toがnullの場合はfrom座標の駒が移動可能なマスがひとつでもあるかどうかを判定)
     *
     * @param fromX
     * @param fromY
     * @param toX
     * @param toY
     */
    isMovable(from: Pos, to?: Pos | null): boolean;
    /**
     * fromで与えられた駒が移動できる座標候補を返す
     * @param from
     */
    getKomaMoves(from: Pos): Array<Array<number>>;
    /**
     * 次の指し手として動かすことのできる駒の座標を返す
     */
    getMovables(): Array<Array<number>>;
    /**
     * 持ち駒から駒を置ける座標情報を返す
     *
     * @param putFU
     */
    getPutables(komaString: string): Array<Array<number>>;
    /**
     * 指定プレイヤーが該当の駒を持っているかどうか
     *
     * @param player
     * @param komaNum
     */
    isInHand(player: number, komaNum: number): boolean;
    readonly initData: IStateFormat;
    /**
     * 最後に指したプレイヤーを返す
     */
    readonly color: number;
    /**
     * 次に指すプレイヤーを返す
     */
    readonly nextColor: number;
    /**
     * 現在の盤面を返す
     */
    readonly board: IPiece[][];
    /**
     * 反転盤面を返す
     */
    readonly reverseBoard: IPiece[][];
    /**
     * 現在のそれぞれの手持ち駒を返す
     */
    readonly hands: {
        [index: string]: number;
    }[];
    /**
     * 盤面の指定座標の盤面情報を返す
     *
     * @param kx
     * @param ky
     */
    getBoardPiece(kx: number, ky: number): IPiece;
    /**
     * 盤面の指定座標に盤面情報で指定された駒を配置する
     *
     * @param pos
     * @param info
     */
    private setBoardPiece;
    /**
     * 指定されたプレイヤーの手駒に駒を追加する
     *
     * @param player
     * @param komaNum
     */
    private addHand;
    /**
     * 指定されたプレイヤーの手駒から駒を削除する
     *
     * @param player
     * @param komaNum
     */
    private deleteHand;
    /**
     * 指定プレイヤーが指定のマスに駒を動かすことができるかどうかを判定
     *
     * @param to
     * @param color
     */
    private isEnterable;
    /**
     * 指定された座標のマスが空かどうか判定する
     *
     * @param boardObj
     */
    private isExists;
    /**
     * 指定位置にその指定種類の駒を配置したとき、移動可能なマスが存在するかどうかを返す
     *
     * @param pos
     * @param komaNum
     * @param color
     * @param nifu 二歩判定を行うかどうか
     */
    private canSet;
}
