import Move from './move';
import { IMoveFormat } from 'json-kifu-format/src/Formats';
export default class MoveNode {
    private _info;
    private _moveObj;
    private _index;
    private _next;
    private _prev;
    private _select;
    private _isBranch;
    /**
     * MoveNodeクラス
     * 指し手のリスト構造におけるひとつのセルを表現する
     *
     * @param moveObj セルの元となるひとつのjson棋譜オブジェクトの指し手オブジェクト
     * @param index  この指し手セルに対して割り当てられるインデックス
     * @param prevIndex この指し手セルの前の指し手を表す指し手セルのインデックス
     */
    constructor(moveObj: IMoveFormat, index: number, prevIndex: number | null, isBranch: boolean);
    readonly prev: number | null;
    readonly next: Array<number>;
    readonly info: Move;
    readonly moveObj: IMoveFormat;
    readonly index: number;
    readonly select: number;
    readonly isBranch: boolean;
    /**
     * 次の指し手候補ノードを追加する
     *
     * @param nextNum 次の指し手となる指し手セルの追加
     */
    addNext(nextNum: number): void;
    /**
     * 指定した指し手候補ノードを削除する
     *
     * @param deleteNum
     */
    deleteNext(deleteNum: number): boolean;
    /**
     * 指し手の分岐を切り替える
     *
     * @param forkIndex 分岐指し手のインデックス
     */
    switchFork(forkIndex: number): boolean;
    swapFork(forkIndex1: number, forkIndex2: number): boolean;
    /**
     * 分岐したひとつの枝であることを示す印をつける
     */
    branchize(): void;
}
